@@ -536,6 +536,7 @@ static JSValue js_c_function(
 	}
 	if (!tuple_success) {
 		Py_DECREF(args);
+		end_call_python(context);
 		return JS_ThrowInternalError(ctx, "Internal error: could not convert args.");
 	}
 
@@ -548,6 +549,9 @@ static JSValue js_c_function(
 	JSValue js_result = JS_NULL;
 	if (python_to_quickjs_possible(context, result)) {
 		js_result = python_to_quickjs(context, result);
+	} else {
+		PyErr_Clear();
+		js_result = JS_ThrowInternalError(ctx, "Can not convert Python result to JS.");
 	}
 	Py_DECREF(result);
 
